##################################################
# Plot function used to generate Figures 1 and 3 #
##################################################

plotfunction<-function(resultsfile1,dir)
{
load(paste(dir,"/",resultsfile1,sep=""))
attach(result)

testmcompetitors[is.na(testmcompetitors[,2])&is.na(testmcompetitors[,2]),2]<-0.5
aucmcompetitors[is.na(aucmcompetitors[,2])&is.na(aucmcompetitors[,2]),2]<-0.5

nw<-ncol(cvm)-1



par(mfrow=c(1,4),cex.main=1.5,cex.axis=1.3,cex.lab=1.3)

# Misclassification rate
boxplot(testm[,nw+1],testm[,1],testmcompetitors[,1],testmcompetitors[,2],
		las=2,col=c(8,0,0,0),
		names=c("IPF","standard","SGL","S"),
		ylab="misclassification rate",ylim=c(0,0.6))

# AUC
boxplot(aucm[,nw+1],aucm[,1],aucmcompetitors[,1],aucmcompetitors[,2],
		las=2,col=c(8,0,0,0),names=c("IPF","standard","SGL","S"),
		ylab="AUC",ylim=c(0.5,1))


if (prel2==0)
 {
 mtext(bquote(paste(
 p[1]==.(p1),", ",
 p[2]==.(p2),", ",
 p[1]^r==.(prel1),", ",
 p[2]^r==.(prel2),", ",
 beta[1]==.(beta1),sep="")),
 line=2.25,cex=1.7,padj=0.8) 
 }

if (prel1==0)
 {
 mtext(bquote(paste(
 p[1]==.(p1),", ",
 p[2]==.(p2),", ",
 p[1]^r==.(prel1),", ",
 p[2]^r==.(prel2),", ",
 beta[2]==.(beta2),sep="")),
 line=2.25,cex=1.7,padj=0.8) 
 }

if (prel1!=0&prel2!=0)
 {
 mtext(bquote(paste(
 p[1]==.(p1),", ",
 p[2]==.(p2),", ",
 p[1]^r==.(prel1),", ",
 p[2]^r==.(prel2),", ",
 beta[1]==.(beta1),", ",
 beta[2]==.(beta2),sep="")),
 line=2.25,cex=1.7,padj=0.8) 
 }


# nzero1, nzerotrue1, nzero2, nzerotrue2

boxplot(nzero[,nw+1],nzero[,1],nzerocompetitors[,1],nzerocompetitors[,2],
		las=2,col=c(8,0,0,0),names=c("IPF","standard","SGL","S"),ylab="number of selected predictors")


# Barplot best penalty factor

barplot(c(sum(bestpf=="1-1"),sum(bestpf=="1-2"),sum(bestpf=="1-4"),sum(bestpf=="1-8"),
	sum(bestpf=="2-1"),sum(bestpf=="4-1"),sum(bestpf=="8-1")),las=2,names=c("1-1","1-2","1-4","1-8","1-1/2","1-1/4","1-1/8"),ylab="frequency",main="selected penalty factors\n with IPF-lasso")
}


###########################################
# Plot function used to generate Figure 2 #
###########################################

plotfunction_additional<-function(dir)
{
settings<-list.files(paste(dir,"/",sep=""))
p<-length(settings)

matrix.settings<-matrix(NA,p,6)
matrix.perf<-matrix(NA,p,3)
matrix.size<-matrix(NA,p,3)

for (j in 1:p)
 {
 load(paste(dir,"/",settings[j],sep="")) 
 attach(result)
 matrix.settings[j,1]<-prel1
 matrix.settings[j,2]<-prel2
 matrix.settings[j,3]<-p1
 matrix.settings[j,4]<-p2
 matrix.settings[j,5]<-beta1
 matrix.settings[j,6]<-beta2
 matrix.perf[j,1]<-median(aucm[,ncol(testm)])
 matrix.perf[j,2]<-median(aucm[,1])
 matrix.perf[j,3]<-median(aucmcompetitors[,1])
 matrix.size[j,1]<-median(nzero[,ncol(nzero)])
 matrix.size[j,2]<-median(nzero[,1])
 matrix.size[j,3]<-median(nzerocompetitors[,1])
 }
matrix.settings<-as.data.frame(matrix.settings)
matrix.perf<-as.data.frame(matrix.perf)
names(matrix.settings)<-c("prel1","prel2","p1","p2","beta1","beta2")
names(matrix.perf)<-c("IPF","standard","SGL")
attach(matrix.settings)
attach(matrix.perf)

par(mfrow=c(2,2),mai=c(0.7,0.7,0.5,0.1))
plot(abs(prel1/p1-prel2/p2),IPF-SGL,xlab="abs. diff. in proportions of relevant predictors",ylim=c(-0.12,0.12),ylab="AUC IPF - AUC competitor",col=1)
points(abs(prel1/p1-prel2/p2),IPF-standard,col=2)
abline(h=0)
legend(x=1,y=-0.05,xjust=1,legend=c("IPF-SGL","IPF-standard"),col=1:2,pch=1)
text(x=0.95,y=0.11,label="A",cex=2.5)

plot(prel1+prel2,IPF-SGL,xlab="true model size",ylim=c(-0.12,0.12),ylab="AUC IPF - AUC competitor",col=1)
points(prel1+prel2,IPF-standard,col=2)
abline(h=0)
legend(x=100,y=-0.05,xjust=1,legend=c("IPF-SGL","IPF-standard"),col=1:2,pch=1)
text(x=95,y=0.11,label="B",cex=2.5)

plot(apply(cbind(p1,p2),MARGIN=1,FUN=min)/apply(cbind(p1,p2),MARGIN=1,FUN=max),IPF-SGL,xlab="min(p1,p2)/max(p1,p2)",ylim=c(-0.12,0.12),ylab="AUC IPF - AUC competitor",col=1)
points(apply(cbind(p1,p2),MARGIN=1,FUN=min)/apply(cbind(p1,p2),MARGIN=1,FUN=max),IPF-standard,col=2)
abline(h=0)
legend(x=1,y=-0.05,xjust=1,legend=c("IPF-SGL","IPF-standard"),col=1:2,pch=1)
text(x=0.95,y=0.11,label="C",cex=2.5)

boxplot(matrix.size[,1],matrix.size[,2],matrix.size[,3],names=c("IPF","standard","SGL"),ylab="number of selected predictors")
text(x=3.35,y=210,label="D",cex=2.5)

return(list(matrix.settings=matrix.settings,matrix.perf=matrix.perf,matrix.size=matrix.size))
}

