# code for the TCGA example
library(snowfall)
sfInit(parallel=T,cpus=50,type='MPI')
library(cgdsr)
sfLibrary(survival)
sfLibrary(ipflasso,lib='./../')
sfLibrary(SGL)
sfLibrary(CoxBoost)
sfLibrary(pec)

# load clinical data
mycgds = CGDS("http://www.cbioportal.org/public-portal/")
cancers<-getCancerStudies(mycgds)

list.laml<-getCaseLists(mycgds,'laml_tcga_pub')
valid<-duplicated(c(unlist(strsplit(list.laml[5,5],' ')),unlist(strsplit(list.laml[7,5],' '))))
exp_snp<-unique(c(unlist(strsplit(list.laml[5,5],' ')),unlist(strsplit(list.laml[7,5],' ')))[valid])
valid<-duplicated(c(exp_snp,unlist(strsplit(list.laml[8,5],' '))))
samples_num<-unique(c(exp_snp,unlist(strsplit(list.laml[8,5],' ')))[valid])
clin.data<-getClinicalData(mycgds,'laml_tcga_pub_all')
rownames(clin.data)<-gsub('.','-',rownames(clin.data),fixed=T)

clinical<-clin.data[,c('AGE','BM_BLAST_PERCENTAGE','FAB','RACE','SEX','WBC')]
clin.data$OS_MONTHS[clin.data$OS_MONTHS<=0]<-1e-9
outcome<-Surv(clin.data$OS_MONTHS,clin.data$OS_STATUS=='DECEASED')
rownames(outcome)<-rownames(clinical)
# remove fab and race because too small group sizes
clinical<-clinical[,-c(3,4)]
# dummy variable for sex
clinical$SEX<-as.numeric(clinical$SEX=='Male')

# load gene expression data, the data must be manually downloaded from the TCGA website
gene.expr<-NULL
for (i in samples_num)
{
  try(tmp<-read.table(paste0('./../../tmp/dataset/fullLAML/Expression-Genes/WUSM__HG-U133_Plus_2/Level_3/',i,'A-01R-0757-21.pergene.txt'),
                      header=F,skip=2,sep='\t')[,2],silent=T)
  if(class(tmp)=='try-error')
  {
    try(tmp<-read.table(paste0('./../../tmp/dataset/fullLAML/Expression-Genes/WUSM__HG-U133_Plus_2/Level_3/',i,'B-01R-0757-21.pergene.txt'),
                                                  header=F,skip=2,sep='\t')[,2],silent=T)
    if(class(tmp)!='try-error') warning(paste0('check sample ',i))
  }
  gene.expr<-rbind(gene.expr,tmp)
  print(i)
}
# NB: the wanrings are related to samples for which gene expression data are not collected: no problem
rownames(gene.expr)<-samples_num
colnames(gene.expr)<-read.table('./../../tmp/dataset/fullLAML/Expression-Genes/WUSM__HG-U133_Plus_2/Level_3/TCGA-AB-2803-03A-01R-0757-21.pergene.txt',
                                header=F,skip=2,sep='\t')[,1]                                
# take the log2
gene.expr<-log2(gene.expr)

### this part of the code was provided by Quin Zhao ###
HUGO <- read.delim("HUGO.txt", as.is=T, sep="\t") # this file can be downloaded from http://www.genenames.org/
approved <- HUGO[HUGO$Status=="Approved", 2]
mycgds <- CGDS("http://www.cbioportal.org/public-portal/")
test(mycgds)
# Get list of cancer studies at server                                                                                                                                      ..........................
getCancerStudies(mycgds)
# leukemia study
mycancerstudy <- getCancerStudies(mycgds)[2, 1]  
# all observations
mycaselist <- getCaseLists(mycgds, mycancerstudy)[2, 1]

# get available genetic profiles
id<-8
mygeneticprofile = getGeneticProfiles(mycgds,mycancerstudy)[id,1]

all <- rep(0, nrow(getProfileData(mycgds, approved[1], mygeneticprofile, mycaselist)))
p = length(approved)
i <- 1
while(i < p){
    result <- try(getProfileData(mycgds, approved[i], mygeneticprofile, mycaselist))
    if(class(result)!="try-error"){
	 	one <- result
	 	if(nrow(one)!=0 & any(!is.na(one))){
	 		all <- cbind(all, one)
	 		print(i)
	 	}
	 	i <- i + 1
    }
}
dim(all)
all <- all[, -1]
index = which(rowSums(is.na(all))!=dim(all)[2])

write.table(all[index,], 'mygeneticprofile.txt', quote = FALSE)
### end of the part of the code provided by Qhin Zhao ###

third<-read.table('mygeneticprofile.txt')
rownames(third)<-gsub('.','-',rownames(third),fixed=T)

# select patients for which all sources are available
valid<-c(rownames(clinical),rownames(gene.expr))[duplicated(c(rownames(clinical),rownames(gene.expr)))]
valid<-c(valid,rownames(third))[duplicated(c(valid,rownames(third)))]

clin.data<-clinical[valid,]
gene.data<-gene.expr[valid,]
three.data<-third[valid,]
outcome<-outcome[valid]

fact.data<-NULL
for (i in 1:ncol(three.data))
{
  fact.data<-cbind(fact.data,as.numeric(sign(three.data[,i])==-1),as.numeric(sign(three.data[,i])==1))
  colnames(fact.data)[(ncol(fact.data)-1):ncol(fact.data)]<-paste0(colnames(three.data)[i],1:2)
  print(i)
}
rownames(fact.data)<-rownames(three.data)

# analyze the data for the single split in training and validation set
single_analysis<-function(seed,outcome,clin.data,gene.data,fact.data)
{
  ncv<-10
  nf<-5
    
  n<-nrow(clin.data)
  set.seed(seed)
  label<-sample(n,round(n*2/3))
  
  # data standardization
  age.mean<-mean(clin.data$AGE[label])
  age.sd<-sd(clin.data$AGE[label])
  clin.data$AGE<-(clin.data$AGE-age.mean)/age.sd
  blast.mean<-mean(clin.data$BM_BLAST_PERCENTAGE[label])
  blast.sd<-sd(clin.data$BM_BLAST_PERCENTAGE[label])
  clin.data$BM_BLAST_PERCENTAGE<-(clin.data$BM_BLAST_PERCENTAGE-blast.mean)/blast.sd
  wbc.mean<-mean(clin.data$WBC[label])
  wbc.sd<-sd(clin.data$WBC[label])
  clin.data$WBC<-(clin.data$WBC-wbc.mean)/wbc.sd
  
  gene.mean<-colMeans(gene.data[label,])
  gene.sd<-apply(gene.data[label,],2,sd)
  gene.data<-sapply(1:ncol(gene.data), function(i,x,m,s) (x[,i]-m[i])/s[i], x=gene.data, m=gene.mean, s=gene.sd)

  coeff<-NULL
  p_tot<-ncol(clin.data)+ncol(gene.data)+ncol(fact.data)
  flag<-list(lasso=0,joint=0,sgl=0,ipf=0)
    
  # standard lasso
  set.seed(seed)
  flag$lasso<-try(cvr.glmnet(X=as.matrix(cbind(clin.data,gene.data,fact.data)[label,]),Y=outcome[label],family='cox',
                             standardize=F,nfolds=nf,ncv=ncv))
  ifelse(class(flag$lasso)=='try-error',coeff$lasso<-rep(0,p_tot),
         coeff$lasso<-flag$lasso$coeff[-1,which.min(flag$lasso$cvm)[1]])
  
  # separate lasso
  set.seed(seed)
  flag1<-try(cvr.glmnet(X=as.matrix(clin.data[label,]),Y=outcome[label],family='cox',standardize=F,nfolds=nf,ncv=ncv))
  set.seed(seed)
  flag2<-try(cvr.glmnet(X=as.matrix(gene.data[label,]),Y=outcome[label],family='cox',standardize=F,nfolds=nf,ncv=ncv))
  set.seed(seed)
  flag3<-try(cvr.glmnet(X=as.matrix(fact.data[label,]),Y=outcome[label],family='cox',standardize=F,nfolds=nf,ncv=ncv))
  if(class(flag1)=='try-error'|class(flag2)=='try-error'|class(flag3)=='try-error')
  {
    class(flag$joint)<-'try-error'
    coeff$joint<-rep(0,p_tot)
  }
  else
  {
    coeff1<-flag1$coeff[,which.min(flag1$cvm)[1]]  
    coeff2<-flag2$coeff[,which.min(flag2$cvm)[1]]  
    coeff3<-flag3$coeff[,which.min(flag3$cvm)[1]]  
    coeff$joint<-c(coeff1[-1],coeff2[-1],coeff3[-1])
  }
  
  # sparse group lasso 
  set.seed(seed)
  flag$sgl<-try(cvSGL(data=list(x=as.matrix(cbind(clin.data,gene.data,fact.data)[label,]),time=outcome[label,1],
                                status=outcome[label,2]),standardize=F,type='cox',
                 index=c(rep(1,ncol(clin.data)),rep(2,ncol(gene.data)),rep(3,ncol(fact.data))),nfold=nf,thresh=0.01))
  ifelse(class(flag$sgl)=='try-error',coeff$sgl<-rep(0,p_tot),coeff$sgl<-flag$sgl$fit$beta[,which.min(flag$sgl$lldiff)[1]])
  
  # IPF-lasso
  pflist<-list(c(1,1,1),c(1,1,2),c(1,2,1),c(1,2,2),c(1,2,4),c(1,2,8),c(1,1,4),c(1,4,1),
               c(1,4,2),c(1,4,4),c(1,4,8),c(1,1,8),c(1,8,1),c(1,8,2),c(1,8,4),c(1,8,8),
               c(2,1,1),c(2,1,2),c(2,2,1),c(2,2,2),c(2,2,4),c(2,2,8),c(2,1,4),c(2,4,1),
               c(2,4,2),c(2,4,4),c(2,4,8),c(2,1,8),c(2,8,1),c(2,8,2),c(2,8,4),c(2,8,8),
               c(4,1,1),c(4,1,2),c(4,2,1),c(4,2,2),c(4,2,4),c(4,2,8),c(4,1,4),c(4,4,1),
               c(4,4,2),c(4,4,4),c(4,4,8),c(4,1,8),c(4,8,1),c(4,8,2),c(4,8,4),c(4,8,8),
               c(8,1,1),c(8,1,2),c(8,2,1),c(8,2,2),c(8,2,4),c(8,2,8),c(8,1,4),c(8,4,1),
               c(8,4,2),c(8,4,4),c(8,4,8),c(8,1,8),c(8,8,1),c(8,8,2),c(8,8,4),c(8,8,8))
  set.seed(seed)
  flag$ipf<-try(cvr2.ipflasso(Y=outcome[label],X=as.matrix(cbind(clin.data,gene.data,fact.data)[label,]),family='cox',
                              standardize=F, blocks=list(block1=1:ncol(clin.data),
                                                         block2=(ncol(clin.data)+1):(ncol(clin.data)+ncol(gene.data)),
                                                         block3=(ncol(clin.data)+ncol(gene.data)+1):p_tot),
                              pflist=pflist,nfolds=nf,ncv=ncv,type.measure='deviance'))
  ifelse(class(flag$ipf)=='try-error',coeff$ipf<-rep(0,p_tot),
         coeff$ipf<-flag$ipf$a[[flag$ipf$ind.bestpf]]$coeff[-1,flag$ipf$ind.bestlambda])
  
  computeModel<-function(coeff,data,outcome)
  {
    colnames<-c('time','status',colnames(data)[coeff!=0])
    data<-data.frame(outcome[,1],outcome[,2],data[,coeff!=0])
    colnames(data)<-colnames
    coxph(Surv(time,status)~.,data=data,init=coeff[coeff!=0],iter.max=0)
  }
  models<-lapply(coeff, computeModel, data=cbind(clin.data,gene.data,fact.data)[label,],outcome=outcome[label])
  
  data.valid<-cbind(outcome[,1],outcome[,2],clin.data,gene.data,fact.data)
  colnames(data.valid)[1:2]<-c('time','status')
  varsUsed<-apply(matrix(unlist(coeff),ncol=ncol(data.valid)-2,byrow=T)!=0,2,sum)>0
  data.valid<-data.valid[-label,c(T,T,varsUsed)]
  
  dim<-lapply(coeff,function(x) sum(x!=0))
  for (i in names(dim)[dim==0]) models[[i]]<-NULL
  predE<-pec(models,Surv(time,status)~1,data=data.valid,times=seq(0,60,le=100),exact=F)
  
  list(predE=predE, dim=dim, flag=lapply(flag,function(x) class(x)=='try-error'))
}

res<-sfSapply(1:100,single_analysis,outcome=outcome,clin.data=clin.data,gene.data=gene.data,fact.data=fact.data)

# read prediction errors
predErr<-list(Reference=NULL, lasso=NULL, joint=NULL, sgl=NULL, ipf=NULL)
ibs<-dim<-flag<-NULL
for (i in 1:100)
{
  tmp<-ibs(res[[1,i]])
  for (mod in c('Reference','lasso','joint','sgl','ipf'))
  {
    # if is null, it means that the model did not select any predictor, i.e. the null model is used
    if(is.null(res[1,i][[1]][[1]][[mod]])) res[1,i][[1]][[1]][[mod]]<-res[1,i][[1]][[1]][[1]]
    # if the validation data do not contain the largest time values, add NA
    notcomp<-100-length(res[1,i][[1]][[1]][[mod]])
    if (notcomp>0) predErr[[mod]]<-rbind(predErr[[mod]],c(res[1,i][[1]][[1]][[mod]],rep(NA,notcomp)))
    else predErr[[mod]]<-rbind(predErr[[mod]],res[1,i][[1]][[1]][[mod]])
    # if is null, it means that the model did not select any predictor, i.e. the null model is used 
    if(class(try(tmp[mod,],silent = T))=='try-error')
    {
      tmp<-rbind(tmp,tmp['Reference',])
      rownames(tmp)[nrow(tmp)]<-mod
    }
  }
  ibs<-rbind(ibs,c(tmp[c('Reference','lasso','joint','sgl','ipf'),]))
  dim<-rbind(dim,unlist(res[[2,i]]))
  flag<-rbind(flag,unlist(res[[3,i]]))
}
colMeans(ibs)
apply(flag,2,sum)
colMeans(dim)

pdf('peCurves_aml.pdf')
toplot<-lapply(predErr,colMeans,na.rm=T)
plot(seq(0,600,le=100),toplot[[1]],type='S',xlab='month',ylab='prediction error (integrated Brier score)')
for (i in 2:4) lines(seq(0,600,le=100),toplot[[i]],type='S',col=i)
lines(seq(0,600,le=100),toplot[[5]],type='S',col=6)
legend('bottomright',c('Kaplan-Meier','standard lasso','separate lasso (S)','sparse group lasso (SGL)', 'IPF-lasso (IPF)'),
       lty=1,col=c(1:4,6))
dev.off()
