library(ipflasso,lib='./..')
library(CoxBoost)
library(glmnet)
library(SGL)
library(pec)
library(GEOquery)

# download the data
GSE25055<-getGEO("GSE25055", GSEMatrix = TRUE)
GSE25065<-getGEO("GSE25065", GSEMatrix = TRUE)

trainData.genes<-t(exprs(GSE25055[[1]]))
validData.genes<-t(exprs(GSE25065[[1]]))
tDc<-pData(phenoData(GSE25055[[1]]))[,10:32]
vDc<-pData(phenoData(GSE25065[[1]]))[,10:32]

trainData.clin<-as.data.frame(matrix(0,dim(tDc)))
trainData.clin$ID<-substr(tDc$characteristics_ch1,12,1e2)
trainData.clin$src<-as.factor(substr(tDc$characteristics_ch1.1,10,1e2))
trainData.clin$age<-as.numeric(substr(tDc$characteristics_ch1.2,12,1e2))
trainData.clin$er<-substr(tDc$characteristics_ch1.3,16,1e2)
trainData.clin$er[trainData.clin$er=='NA']<-NA
trainData.clin$er<-as.factor(trainData.clin$er)
trainData.clin$pr<-substr(tDc$characteristics_ch1.4,16,1e2)
trainData.clin$pr[trainData.clin$pr=='NA']<-NA
trainData.clin$pr<-as.factor(trainData.clin$pr)
trainData.clin$her2<-substr(tDc$characteristics_ch1.5,14,1e2)
trainData.clin$her2[trainData.clin$her2=='NA']<-NA
trainData.clin$her2<-as.factor(trainData.clin$her2)
trainData.clin$er_woI<-as.factor(substr(tDc$characteristics_ch1.6,39,1e2))
trainData.clin$t_stage<-as.factor(substr(tDc$characteristics_ch1.7,19,1e2))
trainData.clin$nodal_status<-as.factor(substr(tDc$characteristics_ch1.8,24,1e2))
trainData.clin$ajcc_stage<-as.factor(substr(tDc$characteristics_ch1.9,22,1e2))
trainData.clin$grade<-substr(tDc$characteristics_ch1.10,8,1e2)
trainData.clin$grade[trainData.clin$grade=='4=Indeterminate']<-NA
trainData.clin$grade[trainData.clin$grade=='NA']<-NA
trainData.clin$grade<-as.factor(trainData.clin$grade)
trainData.clin$pcr_rd<-substr(tDc$characteristics_ch1.11,29,1e2)
trainData.clin$pcr_rd[trainData.clin$pcr_rd=='NA']<-NA
trainData.clin$pcr_rd<-as.factor(trainData.clin$pcr_rd)
trainData.clin$rcb_class<-substr(tDc$characteristics_ch1.12,32,1e2)
trainData.clin$rcb_class[trainData.clin$rcb_class=='NA']<-NA
trainData.clin$rcb_class<-as.factor(trainData.clin$rcb_class)
trainData.clin$drfs_cens<-as.numeric(substr(tDc$characteristics_ch1.13,26,1e2)) # 0 censored, 1 event
trainData.clin$drfs_time<-as.numeric(substr(tDc$characteristics_ch1.14,23,1e2)) # in years
trainData.clin$esr1_status<-as.factor(substr(tDc$characteristics_ch1.15,14,1e2))
trainData.clin$erbb2_status<-as.factor(substr(tDc$characteristics_ch1.16,15,1e2))
trainData.clin$set_class<-as.factor(substr(tDc$characteristics_ch1.17,12,1e2))
trainData.clin$chemosensitivity_prediction<-as.factor(substr(tDc$characteristics_ch1.18,30,1e2))
trainData.clin$ggi_class<-as.factor(substr(tDc$characteristics_ch1.19,12,1e2))
trainData.clin$pam50_class<-as.factor(substr(tDc$characteristics_ch1.20,14,1e2))
trainData.clin$dlda30_prediction<-as.factor(substr(tDc$characteristics_ch1.21,20,1e2))
trainData.clin$rcb_prediction<-as.factor(substr(tDc$characteristics_ch1.22,21,1e2))
trainData.clin<-trainData.clin[,-1]

validData.clin<-as.data.frame(matrix(0,dim(vDc)))
validData.clin$ID<-substr(vDc$characteristics_ch1,12,1e2)
validData.clin$src<-as.factor(substr(vDc$characteristics_ch1.1,10,1e2))
validData.clin$age<-as.numeric(substr(vDc$characteristics_ch1.2,12,1e2))
validData.clin$er<-substr(vDc$characteristics_ch1.3,16,1e2)
validData.clin$er[validData.clin$er=='NA']<-NA
validData.clin$er<-as.factor(validData.clin$er)
validData.clin$pr<-substr(vDc$characteristics_ch1.4,16,1e2)
validData.clin$pr[validData.clin$pr=='NA']<-NA
validData.clin$pr<-as.factor(validData.clin$pr)
validData.clin$her2<-substr(vDc$characteristics_ch1.5,14,1e2)
validData.clin$her2[validData.clin$her2=='NA']<-NA
validData.clin$her2<-as.factor(validData.clin$her2)
validData.clin$t_stage<-as.factor(substr(vDc$characteristics_ch1.6,19,1e2))
validData.clin$nodal_status<-as.factor(substr(vDc$characteristics_ch1.7,24,1e2))
validData.clin$ajcc_stage<-as.factor(substr(vDc$characteristics_ch1.8,22,1e2))
validData.clin$grade<-substr(vDc$characteristics_ch1.9,8,1e2)
validData.clin$grade[validData.clin$grade=='23']<-NA
validData.clin$grade[validData.clin$grade=='NA']<-NA
validData.clin$grade<-as.factor(validData.clin$grade)
validData.clin$pcr_rd<-substr(vDc$characteristics_ch1.10,29,1e2)
validData.clin$pcr_rd[validData.clin$pcr_rd=='NA']<-NA
validData.clin$pcr_rd<-as.factor(validData.clin$pcr_rd)
validData.clin$rcb_class<-substr(vDc$characteristics_ch1.11,32,1e2)
validData.clin$rcb_class[validData.clin$rcb_class=='NA']<-NA
validData.clin$rcb_class<-as.factor(validData.clin$rcb_class)
validData.clin$drfs_cens<-as.numeric(substr(vDc$characteristics_ch1.12,26,1e2)) # 0 censored, 1 event
validData.clin$drfs_time<-as.numeric(substr(vDc$characteristics_ch1.13,23,1e2)) # in years
validData.clin$taxane<-as.factor(substr(vDc$characteristics_ch1.14,14,1e2))
validData.clin$esr1_status<-as.factor(substr(vDc$characteristics_ch1.15,14,1e2))
validData.clin$erbb2_status<-as.factor(substr(vDc$characteristics_ch1.16,15,1e2))
validData.clin$set_class<-as.factor(substr(vDc$characteristics_ch1.17,12,1e2))
validData.clin$chemosensitivity_prediction<-as.factor(substr(vDc$characteristics_ch1.18,30,1e2))
validData.clin$ggi_class<-as.factor(substr(vDc$characteristics_ch1.19,12,1e2))
validData.clin$pam50_class<-as.factor(substr(vDc$characteristics_ch1.20,14,1e2))
validData.clin$dlda30_prediction<-as.factor(substr(vDc$characteristics_ch1.21,20,1e2))
validData.clin$rcb_prediction<-as.factor(substr(vDc$characteristics_ch1.22,21,1e2))
validData.clin<-validData.clin[,-1]

# codify clinical data (training set)
clin.t<-trainData.clin[,c(3,5,7,8,9,11)]
clin.t$er<-clin.t$pr.tmp<-clin.t$T2<-clin.t$T3<-clin.t$T4<-rep(0,nrow(clin.t))
clin.t$N1<-clin.t$N2<-clin.t$N3<-clin.t$G2<-clin.t$G3<-rep(0,nrow(clin.t))
clin.t$er[clin.t$er_woI=='P']<-1
clin.t$pr.tmp[is.na(clin.t$pr)|clin.t$pr=='I']<-NA
clin.t$pr.tmp[clin.t$pr=='P']<-1
clin.t$pr<-clin.t$pr.tmp
clin.t$T2[clin.t$t_stage=='T2']<-1
clin.t$T3[clin.t$t_stage=='T3']<-1
clin.t$T4[clin.t$t_stage=='T4']<-1
clin.t$N1[clin.t$nodal_status=='N1']<-1
clin.t$N2[clin.t$nodal_status=='N2']<-1
clin.t$N3[clin.t$nodal_status=='N3']<-1
clin.t$G2[clin.t$grade=='2']<-1
clin.t$G3[clin.t$grade=='3']<-1
clin.t$G2[is.na(clin.t$grade)]<-clin.t$G3[is.na(clin.t$grade)]<-NA
clin.t<-clin.t[,c(1,11,2,9:7,16:12)]
y.t<-Surv(trainData.clin[,15],trainData.clin[,14])

# codify clinical data (validation set)
clin.v<-validData.clin[,c(3:5,7,8,10)]
clin.v$er.tmp<-clin.v$pr.tmp<-clin.v$T2<-clin.v$T3<-clin.v$T4<-clin.v$N1<-clin.v$N2<-clin.v$N3<-0
clin.v$G2<-clin.v$G3<-rep(0,nrow(clin.v))
clin.v$er.tmp[is.na(clin.v$er)]<-NA
clin.v$er.tmp[clin.v$er=='P']<-1
clin.v$er<-clin.v$er.tmp
clin.v$pr.tmp[is.na(clin.v$pr)]<-NA
clin.v$pr.tmp[clin.v$pr=='P']<-1
clin.v$pr<-clin.v$pr.tmp
clin.v$T2[clin.v$t_stage=='T2']<-1
clin.v$T3[clin.v$t_stage=='T3']<-1
clin.v$T4[clin.v$t_stage=='T4']<-1
clin.v$N1[clin.v$nodal_status=='N1']<-1
clin.v$N2[clin.v$nodal_status=='N2']<-1
clin.v$N3[clin.v$nodal_status=='N3']<-1
clin.v$G2[clin.v$grade=='2']<-1
clin.v$G3[clin.v$grade=='3']<-1
clin.v$G2[is.na(clin.v$grade)]<-clin.v$G3[is.na(clin.v$grade)]<-NA
clin.v<-clin.v[,c(1,2,3,12:7,16:15)]
y.v<-Surv(validData.clin[,14],validData.clin[,13])

# remove rows with NA
noNA.t<-apply(is.na(clin.t),1,sum)==0
noNA.v<-apply(is.na(clin.v),1,sum)==0
y.t<-y.t[noNA.t,]
clin.t<-clin.t[noNA.t,]
gene.t<-trainData.genes[noNA.t,]
y.v<-y.v[noNA.v,]
clin.v<-clin.v[noNA.v,]
gene.v<-validData.genes[noNA.v,]

# standardize the data
age.mean<-mean(clin.t$age)
age.sd<-sd(clin.t$age)
clin.t$age<-(clin.t$age-age.mean)/age.sd
clin.v$age<-(clin.v$age-age.mean)/age.sd
gene.mean<-apply(gene.t,2,mean)
gene.sd<-apply(gene.t,2,sd)
gene.t<-scale(gene.t,scale=T)
gene.v<-sapply(1:ncol(gene.v),
               function(i,xx,means,sds) xx[,i]<-(xx[,i]-means[i])/sds[i],
               xx=gene.v, means=gene.mean,sds=gene.sd)

# change the survival time from 0 to 10^-9 to avoid numerical issues
y.t[6,1]<-1e-9

################
### analysis ###
################
ncv<-10
nfolds<-5
pflist<-list(c(1,1),c(1,2),c(1,4),c(1,8),c(1,16),c(1,32),c(1,64),c(2,1),c(4,1),c(8,1),c(16,1),c(32,1),c(64,1))
coeff<-list(lasso=NULL, joint=NULL, sgl=NULL, ipf=NULL)

# standard lasso
set.seed(1)
mod.lasso<-cvr.glmnet(Y=y.t,X=as.matrix(cbind(clin.t,gene.t)),family='cox',standardize=FALSE,ncv=ncv,nfolds=nfolds)
coeff$lasso<-mod.lasso$coeff[-1,which.min(mod.lasso$cvm)[1]]

# separate lasso
set.seed(1)
model1<-cvr.glmnet(Y=y.t,X=as.matrix(clin.t),family='cox',standardize=FALSE,ncv=ncv,nfolds=nfolds)
set.seed(1)
model2<-cvr.glmnet(Y=y.t,X=as.matrix(gene.t),family='cox',standardize=FALSE,ncv=ncv,nfolds=nfolds)
coeff$joint<-c(model1$coeff[-1,which.min(model1$cvm)[1]],model2$coeff[-1,which.min(model2$cvm)[1]])

# sparse group lasso # 
set.seed(1)
mod.sgl<-cvSGL(data=list(x=as.matrix(cbind(clin.t,gene.t)),time=y.t[,1],status=y.t[,2]),standardize=FALSE,
               index=c(rep(1,ncol(clin.t)),rep(2,ncol(gene.t))),type='cox',nfold=nfolds,thresh=0.01)
coeff$sgl<-mod.sgl$fit$beta[,which.min(mod.sgl$lldiff)[1]]

# IPF-lasso #
set.seed(1)
mod.ipf<-cvr2.ipflasso(Y=y.t,X=as.matrix(cbind(clin.t,gene.t)),family='cox',standardize=FALSE,type.measure='deviance',
                       blocks=list(block1=1:11,block2=12:22294),pflist=pflist,nfolds=nfolds,ncv=ncv)
coeff$ipf<-mod.ipf$a[[mod.ipf$ind.bestpf]]$coeff[-1,mod.ipf$ind.bestlambda]

# different choice for penalties
coeffIPF<-lapply(pflist,
                 function(pf,y.t,x.t,nfolds,nvc)
                 {
                   tmp<-cvr.ipflasso(Y=y.t,X=x.t,family='cox',standardize=FALSE,type.measure='deviance',
                                     blocks=list(block1=1:11,block2=12:22294),pf=pf,nfolds=nfolds,ncv=ncv)
                   tmp$coeff[-1,tmp$ind.bestlambda]
                 },
                 y.t=y.t, x.t=as.matrix(cbind(clin.t,gene.t)), nfolds=nfolds, nvc=ncv)

# performances #
relevant.variables<-(abs(coeff$lasso)+abs(coeff$joint)+abs(coeff$sgl)+abs(coeff$ipf))>0
dataset.train<-cbind(y.t[,1],y.t[,2],cbind(clin.t,gene.t)[,relevant.variables])
colnames(dataset.train)[1:2]<-c('time','status')
dataset.valid<-cbind(y.v[,1],y.v[,2],cbind(clin.v,gene.v)[,relevant.variables])
colnames(dataset.valid)<-colnames(dataset.train)

computeModel<-function(coeff,data,relev) coxph(Surv(time,status)~.,data=data,init=coeff[relev],iter.max=0)
model<-lapply(coeff, computeModel, data=dataset.train, relev=relevant.variables)

predErr<-pec(model,formula=Surv(time,status)~1,data=dataset.valid,maxtime=5)
pdf('peCurves_Hatzis.pdf')
plot(predErr,legend=F,col=c(1:4,6),xlab='year',lwd=1,ylim=c(0,0.25),ylab='prediction error (integrated Brier score)')
legend('bottomright',c('Kaplan-Meier','standard lasso','separate lasso (S)','sparse group lasso (SGL)', 'IPF-lasso (IPF)'),
       lty=1,lwd=2,col=c(1:4,6))
dev.off()

# integrated Brier score
ibs(predErr,times = 5)
# dimensions
lapply(coeff, function(coeff) sum(coeff!=0))


### comparison different pf values
relevant.variables.IPF<-rep(0,length(coeffIPF[[1]]))
for (i in 1:length(coeffIPF)) relevant.variables.IPF<-relevant.variables.IPF+abs(coeffIPF[[i]])
relevant.variables.IPF<-relevant.variables.IPF>0
dataset.train.IPF<-cbind(y.t[,1],y.t[,2],cbind(clin.t,gene.t)[,relevant.variables.IPF])
colnames(dataset.train.IPF)[1:2]<-c('time','status')
dataset.valid.IPF<-cbind(y.v[,1],y.v[,2],cbind(clin.v,gene.v)[,relevant.variables.IPF])
colnames(dataset.valid.IPF)<-colnames(dataset.train.IPF)

modelIPF<-lapply(coeffIPF, computeModel, data=dataset.train.IPF, relev=relevant.variables.IPF)
predErrIPF<-pec(modelIPF,formula=Surv(time,status)~1,data=dataset.valid.IPF,maxtime=5)
pdf('peCurves_Hatzis_IPF.pdf')
plot(predErrIPF,xlab='year',lwd=1,ylim=c(0,0.25))
dev.off()

#integrated Brier score
ibs5<-ibs(predErrIPF,times = 5)
# dimensions
dim<-lapply(coeffIPF, function(coeff) sum(coeff!=0))

# plot ibs ipf
rel.penalities<-matrix(unlist(pflist),ncol=2,byrow=T)
names<-lapply(modelIPF,function(x) names(x$coefficients)[x$coefficients!=0])
nClinCoeff<-unlist(lapply(names, function(x) sum(sapply(x,function(x) nchar(x)<=3))))
nGeneCoeff<-unlist(dim)-nClinCoeff
labels<-apply(rel.penalities,1,function(x) paste(x[1],x[2],sep='-'))
nvar<-paste0('(',nClinCoeff,'-',nGeneCoeff,')')

pdf('relPen.pdf')
plot(ibs5[c(8:2,9:14)],xaxt = 'n',xlab='relative proportion of penalities',ylab='prediction error (integrated Brier score)')
axis(side=1,labels=c(labels[c(7:1)],expression(1-over(1,2)),expression(1-over(1,4)),expression(1-over(1,8)),
                     expression(1-over(1,16)),expression(1-over(1,32)),expression(1-over(1,64))),at=1:13,las=2)
text(1:5,ibs5[8:4]+0.0002,label=nvar[7:3],cex=0.85)
text(6:13,ibs5[c(3:2,9:14)]-0.0003,label=nvar[c(2:1,9:13)],cex=0.85)
dev.off()

save(list=ls(),file='tmp_Hatzis.Rdata')

