library(glmnet)
library(ipflasso,lib='../')
library(SGL)

# load data #
require(GEOquery)
GSE25055<-getGEO("GSE25055", GSEMatrix = TRUE)
GSE25065<-getGEO("GSE25065", GSEMatrix = TRUE)

trainData.genes<-t(exprs(GSE25055[[1]]))
validData.genes<-t(exprs(GSE25065[[1]]))
tDc<-pData(phenoData(GSE25055[[1]]))[,10:32]
vDc<-pData(phenoData(GSE25065[[1]]))[,10:32]

trainData.clin<-as.data.frame(matrix(0,dim(tDc)))
trainData.clin$ID<-substr(tDc$characteristics_ch1,12,1e2)
trainData.clin$src<-as.factor(substr(tDc$characteristics_ch1.1,10,1e2))
trainData.clin$age<-as.numeric(substr(tDc$characteristics_ch1.2,12,1e2))
trainData.clin$er<-substr(tDc$characteristics_ch1.3,16,1e2)
trainData.clin$er[trainData.clin$er=='NA']<-NA
trainData.clin$er<-as.factor(trainData.clin$er)
trainData.clin$pr<-substr(tDc$characteristics_ch1.4,16,1e2)
trainData.clin$pr[trainData.clin$pr=='NA']<-NA
trainData.clin$pr<-as.factor(trainData.clin$pr)
trainData.clin$her2<-substr(tDc$characteristics_ch1.5,14,1e2)
trainData.clin$her2[trainData.clin$her2=='NA']<-NA
trainData.clin$her2<-as.factor(trainData.clin$her2)
trainData.clin$er_woI<-as.factor(substr(tDc$characteristics_ch1.6,39,1e2))
trainData.clin$t_stage<-as.factor(substr(tDc$characteristics_ch1.7,19,1e2))
trainData.clin$nodal_status<-as.factor(substr(tDc$characteristics_ch1.8,24,1e2))
trainData.clin$ajcc_stage<-as.factor(substr(tDc$characteristics_ch1.9,22,1e2))
trainData.clin$grade<-substr(tDc$characteristics_ch1.10,8,1e2)
trainData.clin$grade[trainData.clin$grade=='4=Indeterminate']<-NA
trainData.clin$grade[trainData.clin$grade=='NA']<-NA
trainData.clin$grade<-as.factor(trainData.clin$grade)
trainData.clin$pcr_rd<-substr(tDc$characteristics_ch1.11,29,1e2)
trainData.clin$pcr_rd[trainData.clin$pcr_rd=='NA']<-NA
trainData.clin$pcr_rd<-as.factor(trainData.clin$pcr_rd)
trainData.clin$rcb_class<-substr(tDc$characteristics_ch1.12,32,1e2)
trainData.clin$rcb_class[trainData.clin$rcb_class=='NA']<-NA
trainData.clin$rcb_class<-as.factor(trainData.clin$rcb_class)
trainData.clin$drfs_cens<-as.numeric(substr(tDc$characteristics_ch1.13,26,1e2)) # 0 censored, 1 event
trainData.clin$drfs_time<-as.numeric(substr(tDc$characteristics_ch1.14,23,1e2)) # in years
trainData.clin$esr1_status<-as.factor(substr(tDc$characteristics_ch1.15,14,1e2))
trainData.clin$erbb2_status<-as.factor(substr(tDc$characteristics_ch1.16,15,1e2))
trainData.clin$set_class<-as.factor(substr(tDc$characteristics_ch1.17,12,1e2))
trainData.clin$chemosensitivity_prediction<-as.factor(substr(tDc$characteristics_ch1.18,30,1e2))
trainData.clin$ggi_class<-as.factor(substr(tDc$characteristics_ch1.19,12,1e2))
trainData.clin$pam50_class<-as.factor(substr(tDc$characteristics_ch1.20,14,1e2))
trainData.clin$dlda30_prediction<-as.factor(substr(tDc$characteristics_ch1.21,20,1e2))
trainData.clin$rcb_prediction<-as.factor(substr(tDc$characteristics_ch1.22,21,1e2))
trainData.clin<-trainData.clin[,-1]

validData.clin<-as.data.frame(matrix(0,dim(vDc)))
validData.clin$ID<-substr(vDc$characteristics_ch1,12,1e2)
validData.clin$src<-as.factor(substr(vDc$characteristics_ch1.1,10,1e2))
validData.clin$age<-as.numeric(substr(vDc$characteristics_ch1.2,12,1e2))
validData.clin$er<-substr(vDc$characteristics_ch1.3,16,1e2)
validData.clin$er[validData.clin$er=='NA']<-NA
validData.clin$er<-as.factor(validData.clin$er)
validData.clin$pr<-substr(vDc$characteristics_ch1.4,16,1e2)
validData.clin$pr[validData.clin$pr=='NA']<-NA
validData.clin$pr<-as.factor(validData.clin$pr)
validData.clin$her2<-substr(vDc$characteristics_ch1.5,14,1e2)
validData.clin$her2[validData.clin$her2=='NA']<-NA
validData.clin$her2<-as.factor(validData.clin$her2)
validData.clin$t_stage<-as.factor(substr(vDc$characteristics_ch1.6,19,1e2))
validData.clin$nodal_status<-as.factor(substr(vDc$characteristics_ch1.7,24,1e2))
validData.clin$ajcc_stage<-as.factor(substr(vDc$characteristics_ch1.8,22,1e2))
validData.clin$grade<-substr(vDc$characteristics_ch1.9,8,1e2)
validData.clin$grade[validData.clin$grade=='23']<-NA
validData.clin$grade[validData.clin$grade=='NA']<-NA
validData.clin$grade<-as.factor(validData.clin$grade)
validData.clin$pcr_rd<-substr(vDc$characteristics_ch1.10,29,1e2)
validData.clin$pcr_rd[validData.clin$pcr_rd=='NA']<-NA
validData.clin$pcr_rd<-as.factor(validData.clin$pcr_rd)
validData.clin$rcb_class<-substr(vDc$characteristics_ch1.11,32,1e2)
validData.clin$rcb_class[validData.clin$rcb_class=='NA']<-NA
validData.clin$rcb_class<-as.factor(validData.clin$rcb_class)
validData.clin$drfs_cens<-as.numeric(substr(vDc$characteristics_ch1.12,26,1e2)) # 0 censored, 1 event
validData.clin$drfs_time<-as.numeric(substr(vDc$characteristics_ch1.13,23,1e2)) # in years
validData.clin$taxane<-as.factor(substr(vDc$characteristics_ch1.14,14,1e2))
validData.clin$esr1_status<-as.factor(substr(vDc$characteristics_ch1.15,14,1e2))
validData.clin$erbb2_status<-as.factor(substr(vDc$characteristics_ch1.16,15,1e2))
validData.clin$set_class<-as.factor(substr(vDc$characteristics_ch1.17,12,1e2))
validData.clin$chemosensitivity_prediction<-as.factor(substr(vDc$characteristics_ch1.18,30,1e2))
validData.clin$ggi_class<-as.factor(substr(vDc$characteristics_ch1.19,12,1e2))
validData.clin$pam50_class<-as.factor(substr(vDc$characteristics_ch1.20,14,1e2))
validData.clin$dlda30_prediction<-as.factor(substr(vDc$characteristics_ch1.21,20,1e2))
validData.clin$rcb_prediction<-as.factor(substr(vDc$characteristics_ch1.22,21,1e2))
validData.clin<-validData.clin[,-1]

x.clin<-trainData.clin[,c(3,5,7,8,9,11)]
x.clin.new<-validData.clin[,c(3,4,5,7,8,10)]
y<-NULL
y[trainData.clin$rcb_class=='RCB-0/I']<-0
y[trainData.clin$rcb_class=='RCB-II'|trainData.clin$rcb_class=='RCB-III']<-1
y[is.na(trainData.clin$rcb_class)]<-NA

y.new<-NULL
y.new[validData.clin$rcb_class=='RCB-0/I']<-0
y.new[validData.clin$rcb_class=='RCB-II'|validData.clin$rcb_class=='RCB-III']<-1
y.new[is.na(validData.clin$rcb_class)]<-NA

noNA.train<-apply(is.na(cbind(y,x.clin)),1,sum)==0
noNA.valid<-apply(is.na(cbind(y.new,x.clin.new)),1,sum)==0

x.clin<-x.clin[noNA.train,]
x.gene<-trainData.genes[noNA.train,]
y<-y[noNA.train]
# delete row with indeterminate pr
valid_pr<-x.clin$pr!='I'
x.clin<-x.clin[valid_pr,]
x.clin$pr<-as.factor(as.character(x.clin$pr))
x.gene<-x.gene[valid_pr,]
y<-y[valid_pr]

x.clin.new<-x.clin.new[noNA.valid,]
x.gene.new<-validData.genes[noNA.valid,]
y.new<-y.new[noNA.valid]

# codify clinical data (training set)
x.clin$er<-x.clin$pr.tmp<-x.clin$T2<-x.clin$T3<-x.clin$T4<-x.clin$N1<-x.clin$N2<-x.clin$N3<-x.clin$G2<-x.clin$G3<-rep(0,nrow(x.clin))
x.clin$er[x.clin$er_woI=='P']<-1
x.clin$pr.tmp[is.na(x.clin$pr)|x.clin$pr=='I']<-NA
x.clin$pr.tmp[x.clin$pr=='P']<-1
x.clin$pr<-x.clin$pr.tmp
x.clin$T2[x.clin$t_stage=='T2']<-1
x.clin$T3[x.clin$t_stage=='T3']<-1
x.clin$T4[x.clin$t_stage=='T4']<-1
x.clin$N1[x.clin$nodal_status=='N1']<-1
x.clin$N2[x.clin$nodal_status=='N2']<-1
x.clin$N3[x.clin$nodal_status=='N3']<-1
x.clin$G2[x.clin$grade=='2']<-1
x.clin$G3[x.clin$grade=='3']<-1
x.clin$G2[is.na(x.clin$grade)]<-x.clin$G3[is.na(x.clin$grade)]<-NA
x.clin<-x.clin[,c(1,16,2,14:7)]

# codify clinical data (validation set)
x.clin.new$er.tmp<-x.clin.new$pr.tmp<-x.clin.new$T2<-x.clin.new$T3<-x.clin.new$T4<-x.clin.new$N1<-x.clin.new$N2<-x.clin.new$N3<-0
x.clin.new$G2<-x.clin.new$G3<-rep(0,nrow(x.clin.new))
x.clin.new$er.tmp[is.na(x.clin.new$er)]<-NA
x.clin.new$er.tmp[x.clin.new$er=='P']<-1
x.clin.new$er<-x.clin.new$er.tmp
x.clin.new$pr.tmp[is.na(x.clin.new$pr)]<-NA
x.clin.new$pr.tmp[x.clin.new$pr=='P']<-1
x.clin.new$pr<-x.clin.new$pr.tmp
x.clin.new$T2[x.clin.new$t_stage=='T2']<-1
x.clin.new$T3[x.clin.new$t_stage=='T3']<-1
x.clin.new$T4[x.clin.new$t_stage=='T4']<-1
x.clin.new$N1[x.clin.new$nodal_status=='N1']<-1
x.clin.new$N2[x.clin.new$nodal_status=='N2']<-1
x.clin.new$N3[x.clin.new$nodal_status=='N3']<-1
x.clin.new$G2[x.clin.new$grade=='2']<-1
x.clin.new$G3[x.clin.new$grade=='3']<-1
x.clin.new$G2[is.na(x.clin.new$grade)]<-x.clin.new$G3[is.na(x.clin.new$grade)]<-NA
x.clin.new<-x.clin.new[,c(1,2,3,12:7,16:15)]

# standardize data #
age.mean<-mean(x.clin$age)
age.sd<-sd(x.clin$age)
x.clin$age<-(x.clin$age-age.mean)/age.sd
x.clin.new$age<-(x.clin.new$age-age.mean)/age.sd
gene.mean<-apply(x.gene,2,mean)
gene.sd<-apply(x.gene,2,sd)
x.gene<-scale(x.gene, center = T, scale = T)
x.gene.new<-sapply(1:ncol(x.gene.new),
                    function(i,xx,means,sds) xx[,i]<-(xx[,i]-means[i])/sds[i],
                    xx=x.gene.new, means=gene.mean, sds=gene.sd)

rm(age.mean, age.sd, gene.mean, gene.sd)

###################
ncv<-10
nf<-5
family='binomial'

n<-length(y)
coeff<-list()
  
# naive lasso
set.seed(1)
mod.lasso<-cvr.glmnet(X=as.matrix(cbind(x.clin,x.gene)),Y=y,family=family,standardize=F,nfolds=nf,ncv=ncv)
coeff$lasso<-mod.lasso$coeff[,which.min(mod.lasso$cvm)[1]]
  
# different models
set.seed(1)
mod.1<-cvr.glmnet(X=as.matrix(x.clin),Y=y,family=family,standardize=F,nfolds=nf,ncv=ncv)
set.seed(1)
mod.2<-cvr.glmnet(X=x.gene,Y=y,family=family,standardize=F,nfolds=nf,ncv=ncv)
coeff1<-mod.1$coeff[,which.min(mod.1$cvm)[1]]  
coeff2<-mod.2$coeff[,which.min(mod.2$cvm)[1]]  
coeff$joint<-c(mean(coeff1[1],coeff2[1]),coeff1[-1],coeff2[-1])
  
# sparse group lasso 
set.seed(1)
mod.sgl<-cvSGL(data=list(x=as.matrix(cbind(rep(1,n),x.clin,x.gene)),y=y),standardize=F,type='logit',
               index=c(0,rep(1,ncol(x.clin)),rep(2,ncol(x.gene))),nfold=nf,thresh=0.01)
coeff$sgl<-mod.sgl$fit$beta[,which.min(mod.sgl$lldiff)[1]]
  
# ipf-lasso
pflist<-list(c(1,1),c(1,2),c(1,4),c(1,8),c(1,16),c(2,1),c(4,1),c(8,1),c(16,1),c(1,32),c(1,64))

set.seed(1)
mod.ipf<-cvr2.ipflasso(Y=y,X=as.matrix(cbind(x.clin,x.gene)),family=family,standardize=F,
                       blocks=list(block1=1:ncol(x.clin),block2=(ncol(x.clin)+1):(ncol(x.clin)+ncol(x.gene))),
                       pflist=pflist,nfolds=nf,ncv=ncv,type.measure='class')
coeff$ipf<-mod.ipf$a[[mod.ipf$ind.bestpf]]$coeff[,mod.ipf$ind.bestlambda]

compAUC<-function(coeff,newdata,y)
{
  linpred<-newdata%*%coeff
  my.auc(linpred,y)
}
  
# auc
lapply(coeff,compAUC,newdata=as.matrix(cbind(rep(1,length(y.new)),x.clin.new,x.gene.new)),y=y.new)
# dim 
lapply(coeff,function(x) sum(x!=0))







